//******************************************************************************
//
// SkinnedComponents
//
// Set of Component with skins
//
// TSkinnedPanel:     Panel with skin; skin is implemented by 9 bitmaps
//                    Different Panels can have different skins!
// TSkinnedComponent: Base class for further skinned components (as SkinnedButton)
//
// Date of creation: 28-jul-04, Martin Lacher, martin@martinlacher.ch
//
//******************************************************************************
unit SkinnedComponents;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Math,
  StdCtrls, ExtCtrls, GR32_Image, GR32, GR32_Filters, GR32_Layers, inifiles;

resourcestring
  SListIndexError = 'Der Index der Liste berschreitet das Maximum (%d)';

type
  TSkin = class(TComponent)
  private
    FTopLeft: TBitmap32;
    FTop: TBitmap32;
    FTopRight: TBitmap32;
    FRight: TBitmap32;
    FBottomRight: TBitmap32;
    FBottom: TBitmap32;
    FBottomLeft: TBitmap32;
    FLeft: TBitmap32;
    FClient: TBitmap32;
    FSkinDir: string;
    procedure LoadBitmapFromIni(ini: TiniFile; strSection: string; var bmp: TBitmap32);
  public
    constructor Create(AOwner: TComponent); override;  //new constructor
    destructor Destroy; override;
    procedure LoadFromFile(strFile: string);
    property TopLeft: TBitmap32 read FTopLeft write FTopLeft;
    property Top: TBitmap32 read FTop write FTop;
    property TopRight: TBitmap32 read FTopRight write FTopRight;
    property Right: TBitmap32 read FRight write FRight;
    property BottomRight: TBitmap32 read FBottomRight write FBottomRight;
    property Bottom: TBitmap32 read FBottom write FBottom;
    property BottomLeft: TBitmap32 read FBottomLeft write FBottomLeft;
    property Left: TBitmap32 read FLeft write FLeft;
    property Client: TBitmap32 read FClient write FClient;
    property SkinDir: string read FSkinDir write FSkinDir;
  end;

  TSkinnedComponent = class;

  TSkinnedPanel = class(TPanel)
  private
    FSkin: TSkin;
    FImage32: TImage32;
    FSkinFile: String;
    FLayerIndexTopLeft: integer;
    FLayerIndexTop: integer;
    FLayerIndexTopRight: integer;
    FLayerIndexRight: integer;
    FLayerIndexBottomRight: integer;
    FLayerIndexBottom: integer;
    FLayerIndexBottomLeft: integer;
    FLayerIndexLeft: integer;
    FLayeredComponents: TList;
    function GetLayeredComponent(AIndex: Integer): TSkinnedComponent;
    function GetLayeredComponentCount: Integer;
    procedure InitialiseSkin;
    procedure SetSkinFile(strFile: string);
    procedure AssignBitmapToLayer(LayerIndex: integer; InsBitmap:TBitmap32; Loc: TFloatRect);
    procedure WMWindowPosChanged(var Message: TWMWindowPosChanged); message WM_WINDOWPOSCHANGED;
  protected
    procedure Resize; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;  //new constructor
    destructor Destroy; override;
    procedure Loaded; override;                        //load skin
    procedure InsertLayeredComponent(AComponent: TSkinnedComponent);
    procedure RemoveLayeredComponent(AComponent: TSkinnedComponent);
    procedure SetLayerLocation(LayerIndex: integer; Loc: TFloatRect);   //set location of a layer
    procedure NotifyComponents(FromIndex, ToIndex: integer); //notify other components that layer have been removed (to adjust their layer indexes)
    function InsertBitmapLayer(InsBitmap:TBitmap32; Loc: TFloatRect):integer; overload;  //insert a bitmap to a new layer
    property Skin: TSkin read FSkin;
    property Image32: TImage32 read FImage32;
    property LayeredComponents[Index: Integer]: TSkinnedComponent read GetLayeredComponent;
    property LayeredComponentCount: Integer read GetLayeredComponentCount;
  published
    property SkinFile: string read FSkinFile write SetSkinFile;
  end;

  TSkinnedComponent = class(TGraphicControl)
  private
    FOverComponentThreshold: byte;
    FSkinnedPanel: TSkinnedPanel;
    FAlpha: TBitmap;
    FMouseOver: Boolean;
    FMouseEnter: Boolean;
    FMouseLeave: Boolean;
    FOnMouseEnter: TMouseEvent;
    FOnMouseLeave: TMouseEvent;
    procedure SetSkinnedPanel(NewPanel: TSkinnedPanel);
    procedure SetAlpha(bmp: TBitmap);
  protected
    function GetLocationToPanel:TFloatRect;          //Get location of component referring to panel coords
    procedure MakeBitmap32(var Bitmap32: TBitmap32; Bmp, AlphaBmp: TBitmap); overload; //make in Bitmap32 a bitmap with Bmp and alpha channel AlphaBmp
    procedure MakeBitmap32(var Bitmap32: TBitmap32; Bmp: TBitmap; AlphaBmp: TBitmap32); overload; //make in Bitmap32 a bitmap with Bmp and alpha channel AlphaBmp
    function InsertLayerToPanel(Bitmap: TBitmap; Alpha: TBitmap = nil):integer; //paint button to SkinnedPanel, return layer index
    function IsOverComponent(X,Y : integer):boolean; virtual;  //check if point x/y is over component (via FAlpha)
    procedure SetEnabled(Value: boolean); override;
    procedure Resize; override;
    procedure Loaded; override;
    procedure PanelMouseDown(Sender: TObject; Button: TMouseButton ; Shift: TShiftState; X, Y: Integer); virtual; //event procs for SkinnedPanel layer, assigned in PaintToPanel
    procedure PanelMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
    procedure PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); virtual;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;            //"normal" event proc
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure RemoveFromPanel; virtual;
    procedure InsertToPanel; virtual;
    procedure MakePanelNotify; virtual; abstract; //must be defined by derived class: make panel notify its skinned component that layers has been removed
    property MouseOver: Boolean read FMouseOver write FMouseOver;
    property MouseEnter: Boolean read FMouseEnter write FMouseEnter;
    property MouseLeave: Boolean read FMouseLeave write FMouseLeave;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure PanelMouseHit(Sender: TObject; X, Y: Integer; var Passed: Boolean);  //necessary for SkinnedPanel, check if mouse hit was on button
    procedure NotifyLayersRemoved(FromIndex, ToIndex: integer); virtual; abstract;  //must be defined by derived class
  published
    property Alpha: TBitmap read FAlpha write SetAlpha;
    property OverComponentThreshold: byte read FOverComponentThreshold write FOverComponentThreshold;
    property SkinnedPanel: TSkinnedPanel read FSkinnedPanel write SetSkinnedPanel;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseEnter: TMouseEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TMouseEvent read FOnMouseLeave write FOnMouseLeave;
    property Enabled;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('SkinnedComponents', [TSkinnedPanel]);
end;


//****************************** TSkin *****************************************

(*******************************************************************************
  Create

  constructor
  create other subcomponents:
  FTopLeft, FTop, FTopRight, FRight, FBottomRight, FBottom, FBottomLeft, FLeft, FClient

*******************************************************************************)
constructor TSkin.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  try
    FTopLeft:=TBitmap32.Create;
    FTop:=TBitmap32.Create;
    FTopRight:=TBitmap32.Create;
    FRight:=TBitmap32.Create;
    FBottomRight:=TBitmap32.Create;
    FBottom:=TBitmap32.Create;
    FBottomLeft:=TBitmap32.Create;
    FLeft:=TBitmap32.Create;
    FClient:=TBitmap32.Create;

  except
    on E: Exception do MessageDlg(E.Message,mtInformation,[mbOK],0);
  end;
end; //Create ******************************************************************


(*******************************************************************************
  Destroy

  destructor
*******************************************************************************)
destructor TSkin.Destroy;
begin
  FTopLeft.Free;
  FTop.Free;
  FTopRight.Free;
  FRight.Free;
  FBottomRight.Free;
  FBottom.Free;
  FBottomLeft.Free;
  FLeft.Free;
  FClient.Free;

  inherited Destroy;
end; //Destroy *****************************************************************


(*******************************************************************************
  LoadBitmapFromIni(ini: TiniFile, strSection: string, var bmp: TBitmap32)

  Load skin bitmap and alpha channel from inifile

  return: Bitmap32 in bmp
*******************************************************************************)
procedure TSkin.LoadBitmapFromIni(ini: TiniFile; strSection: string; var bmp: TBitmap32);
var strBitmapFile: string;
    bmpAlpha: TBitmap32;
begin
  strBitmapFile:=ini.ReadString(strSection,'Bitmap','');     //get bitmap filename
  if strBitmapFile<>'' then
  begin
    bmp.LoadFromFile(FSkinDir + strBitmapFile);                         //load bitmap
  end
  else raise EInOutError.Create('Kein Bitmap fr ' + strSection + ' vorhanden.');
  strBitmapFile:=ini.ReadString(strSection,'BitmapAlpha','');  //get bitmap alpha filename
  if strBitmapFile<>'' then
  begin
    bmpAlpha:=TBitmap32.Create;
    bmpAlpha.LoadFromFile(FSkinDir + strBitmapFile);                      //get alpha bitmap
    bmp.ResetAlpha;
    IntensityToAlpha(bmp, bmpAlpha);                           //apply alpha to actual bitmap
    bmpAlpha.Free;
  end;
end; //LoadBitmapFromIni *******************************************************


(*******************************************************************************
  LoadFromFile

  Load skin bitmaps from ini-file
*******************************************************************************)
procedure TSkin.LoadFromFile(strFile: string);
var ini: TiniFile;
begin
  ini := TIniFile.Create(strFile);

  LoadBitmapFromIni(ini,'TopLeft',Self.FTopLeft);
  LoadBitmapFromIni(ini,'Top',Self.FTop);
  LoadBitmapFromIni(ini,'TopRight',Self.FTopRight);
  LoadBitmapFromIni(ini,'Right',Self.FRight);
  LoadBitmapFromIni(ini,'BottomRight',Self.FBottomRight);
  LoadBitmapFromIni(ini,'Bottom',Self.FBottom);
  LoadBitmapFromIni(ini,'BottomLeft',Self.FBottomLeft);
  LoadBitmapFromIni(ini,'Left',Self.FLeft);
  LoadBitmapFromIni(ini,'Client',Self.FClient);

  ini.Free;

end; //LoadFromFile ************************************************************


//**************************** TSkinnedPanel ***********************************

(*******************************************************************************
  InsertBitmapLayer

  Insert a layer on the Image32-component and copy Bitmap32 to it
  return index of layer
*******************************************************************************)
function TSkinnedPanel.InsertBitmapLayer(InsBitmap:TBitmap32; Loc: TFloatRect):integer;
var ALayer: TBitmapLayer;
begin
  FImage32.BeginUpdate;
  ALayer:=TBitmapLayer(FImage32.Layers.Add(TBitmapLayer));
  with ALayer do
  begin
    Bitmap:=InsBitmap;
    Bitmap.DrawMode:=dmBlend;
    Bitmap.MasterAlpha:=255;
    Bitmap.StretchFilter:=sfMitchell;
    Location:=Loc;
    Result:=Index; //return index of layer
  end;
  FImage32.EndUpdate;
end; //InsertBitmapLayer *******************************************************


(*******************************************************************************
  AssignBitmapToLayer

  Assign a bitmap and location to an already created layer
*******************************************************************************)
procedure TSkinnedPanel.AssignBitmapToLayer(LayerIndex: integer; InsBitmap:TBitmap32; Loc: TFloatRect);
begin
  FImage32.BeginUpdate;
  with TBitmapLayer(FImage32.Layers[LayerIndex]) do
  begin
    Bitmap:=InsBitmap;
    Bitmap.DrawMode:=dmBlend;
    Bitmap.MasterAlpha:=255;
    Location:=Loc;
  end;
  FImage32.EndUpdate;
end; //AssignBitmapToLayer *****************************************************


(*******************************************************************************
  Create

  constructor
  create other subcomponents:
  FSkin
*******************************************************************************)
constructor TSkinnedPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  try
    FSkin:=TSkin.Create(Self);
    FImage32:=TImage32.Create(Self);
    FImage32.Parent:=Self;
    FImage32.Align:=alClient;

    //insert all layers for skin. necessary in constructor before eventually SkinnedButton inserts its layers
    FLayerIndexTopLeft:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));
    FLayerIndexTop:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));
    FLayerIndexTopRight:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));
    FLayerIndexRight:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));
    FLayerIndexBottomRight:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));
    FLayerIndexBottom:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));
    FLayerIndexBottomLeft:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));
    FLayerIndexLeft:=InsertBitmapLayer(nil, FloatRect(0,0,0,0));

  except
    on E: Exception do MessageDlg(E.Message,mtInformation,[mbOK],0);
  end;
end; //Create ******************************************************************



(*******************************************************************************
  Destroy

  destructor
*******************************************************************************)
destructor TSkinnedPanel.Destroy;
begin
  FSkin.Free;
  FImage32.Free;

  inherited Destroy;
end; //Destroy *****************************************************************

(*******************************************************************************
  InitialiseSkin

  load and initialise skin
*******************************************************************************)
procedure TSkinnedPanel.InitialiseSkin;
begin
  FSkin.LoadFromFile(FSkinFile);
  //Client
  FImage32.Bitmap:=FSkin.Client;
  FImage32.BitmapAlign:=baTopLeft;
  FImage32.ScaleMode:=smStretch;
  FImage32.Bitmap.StretchFilter:=sfNearest;

  //Assign now all skin bitmaps to already created layers (see constructor create)
  AssignBitmapToLayer(FLayerIndexTopLeft,FSkin.TopLeft, FloatRect(0,0,FSkin.TopLeft.Width,FSkin.TopLeft.Height));
  AssignBitmapToLayer(FLayerIndexTop,FSkin.Top, FloatRect(FSkin.TopLeft.Width,0,Self.Width-FSkin.TopRight.Width,FSkin.Top.Height));
  AssignBitmapToLayer(FLayerIndexTopRight,FSkin.TopRight, FloatRect(Self.Width-FSkin.TopRight.Width,0, Self.Width, FSkin.TopRight.Height));
  AssignBitmapToLayer(FLayerIndexRight,FSkin.Right, FloatRect(Self.Width-FSkin.Right.Width,FSkin.TopRight.Height, Self.Width, Self.Height-FSkin.BottomRight.Height));
  AssignBitmapToLayer(FLayerIndexBottomRight,FSkin.BottomRight, FloatRect(Self.Width-FSkin.BottomRight.Width,Self.Height-FSkin.BottomRight.Height, Self.Width, Self.Height));
  AssignBitmapToLayer(FLayerIndexBottom,FSkin.Bottom, FloatRect(FSkin.BottomLeft.Width,Self.Height-FSkin.Bottom.Height, Self.Width-FSkin.BottomRight.Width, Self.Height));
  AssignBitmapToLayer(FLayerIndexBottomLeft,FSkin.BottomLeft, FloatRect(0,Self.Height-FSkin.BottomLeft.Height, FSkin.BottomLeft.Width, Self.Height));
  AssignBitmapToLayer(FLayerIndexLeft,FSkin.Left, FloatRect(0,FSkin.TopLeft.Height, FSkin.Left.Width, Self.Height-FSkin.BottomLeft.Height));

  FImage32.Visible:=True;
  FImage32.Changed;
end; //InitialiseSkin **********************************************************


(*******************************************************************************
  Loaded

  load skin
*******************************************************************************)
procedure TSkinnedPanel.Loaded;
begin
  inherited Loaded;

  InitialiseSkin;
end; //Loaded ******************************************************************


(*******************************************************************************
  SetSkinFile

  Set skin file and load all associated bitmaps
  FSkin
*******************************************************************************)
procedure TSkinnedPanel.SetSkinFile(strFile: string);
begin
  if FileExists(strFile) then
  begin
    FSkinFile:=strFile;
    Skin.SkinDir:=ExtractFilePath(strFile);
    InitialiseSkin; //load and init skin
  end
  else if strFile<>'' then raise EInOutError.Create('Ungltiges Skin-File');
end; //SetSkinFile *************************************************************


(*******************************************************************************
  SetLayerLocation(LayerIndex, Loc)

  Set the location of a layer
*******************************************************************************)
procedure TSkinnedPanel.SetLayerLocation(LayerIndex: integer; Loc: TFloatRect);
begin
  if FImage32.Layers.Count>LayerIndex then
  begin
    FImage32.BeginUpdate;
    TBitmapLayer(FImage32.Layers[LayerIndex]).Location:=Loc;
    FImage32.EndUpdate;
  end;
end; //SetLayerLocation ********************************************************


(*******************************************************************************
  Resize

  Adjust bitmapped layers and paint image32
*******************************************************************************)
procedure TSkinnedPanel.Resize;
begin
  inherited Resize;

  //adjust layers
  SetLayerLocation(FLayerIndexTopLeft, FloatRect(0,0,FSkin.TopLeft.Width,FSkin.TopLeft.Height));
  SetLayerLocation(FLayerIndexTop, FloatRect(FSkin.TopLeft.Width,0,Self.Width-FSkin.TopRight.Width+1,FSkin.Top.Height));
  SetLayerLocation(FLayerIndexTopRight,FloatRect(Self.Width-FSkin.TopRight.Width,0, Self.Width, FSkin.TopRight.Height));
  SetLayerLocation(FLayerIndexRight,FloatRect(Self.Width-FSkin.Right.Width,FSkin.TopRight.Height, Self.Width, Self.Height-FSkin.BottomRight.Height+1));
  SetLayerLocation(FLayerIndexBottomRight,FloatRect(Self.Width-FSkin.BottomRight.Width,Self.Height-FSkin.BottomRight.Height, Self.Width, Self.Height));
  SetLayerLocation(FLayerIndexBottom,FloatRect(FSkin.BottomLeft.Width,Self.Height-FSkin.Bottom.Height, Self.Width-FSkin.BottomRight.Width+1, Self.Height));
  SetLayerLocation(FLayerIndexBottomLeft,FloatRect(0,Self.Height-FSkin.BottomLeft.Height, FSkin.BottomLeft.Width, Self.Height));
  SetLayerLocation(FLayerIndexLeft,FloatRect(0,FSkin.TopLeft.Height, FSkin.Left.Width, Self.Height-FSkin.BottomLeft.Height+1));
end; //Resize ******************************************************************

(*******************************************************************************
  Paint

  Repaint also Image32
*******************************************************************************)
procedure TSkinnedPanel.Paint;
begin
  inherited Paint;

  FImage32.Invalidate;
end; //Resize ******************************************************************


//to assure correct resize during design time
procedure TSkinnedPanel.WMWindowPosChanged(var Message: TWMWindowPosChanged);
begin
  Resize;
  inherited;
end;


(*******************************************************************************
  NotifyComponents

  Notify all components which use layers in this panel,
  that some layers have been removed (because another layered component has been
  removed).
*******************************************************************************)
procedure TSkinnedPanel.NotifyComponents(FromIndex, ToIndex: integer); //notify other components that layer have been removed (to adjust their layer indexes)
var MaxIndex, MinIndex,i: integer;
begin
  MaxIndex:=Max(FromIndex, ToIndex);
  MinIndex:=Min(FromIndex, ToIndex);

  if LayeredComponentCount>0 then      //notify all layered components, that layers have been removed
    for i:=0 to LayeredComponentCount-1 do LayeredComponents[i].NotifyLayersRemoved(MinIndex, MaxIndex);
end;


function TSkinnedPanel.GetLayeredComponent(AIndex: Integer): TSkinnedComponent;
begin
  if FLayeredComponents = nil then TList.Error(@SListIndexError, AIndex);
  Result := FLayeredComponents[AIndex];
end;

function TSkinnedPanel.GetLayeredComponentCount: Integer;
begin
  if FLayeredComponents <> nil then
    Result := FLayeredComponents.Count else
    Result := 0;
end;

procedure TSkinnedPanel.InsertLayeredComponent(AComponent: TSkinnedComponent);
begin
  if FLayeredComponents = nil then FLayeredComponents := TList.Create;
  FLayeredComponents.Add(AComponent);
end;

procedure TSkinnedPanel.RemoveLayeredComponent(AComponent: TSkinnedComponent);
begin
  FLayeredComponents.Remove(AComponent);
  if FLayeredComponents.Count = 0 then
  begin
    FLayeredComponents.Free;
    FLayeredComponents := nil;
  end;
end;

//************************** TSkinnedComponent *********************************

(*******************************************************************************
  Create

  constructor
*******************************************************************************)
constructor TSkinnedComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FAlpha:=TBitmap.Create;
  FOverComponentThreshold:=127;

  Enabled:=True;
end; //Create ******************************************************************


(*******************************************************************************
  Destroy

  destructor
*******************************************************************************)
destructor TSkinnedComponent.Destroy;
begin
  FAlpha.Free;
  SkinnedPanel:=nil;

  inherited Destroy;
end; //Destroy *****************************************************************

(*******************************************************************************
  GetLocationToPanel

  get the location of the component referring to the associated SkinnedPanel

  return: TFloatRect coordinates
*******************************************************************************)
function TSkinnedComponent.GetLocationToPanel:TFloatRect;
var Rect: TRect;
begin
  Rect.Left:=Self.Left;
  Rect.Top:=Self.Top;
  Rect.Right:=Rect.Left+Self.Width;
  Rect.Bottom:=Rect.Top+Self.Height;
  Rect.TopLeft:=FSkinnedPanel.ScreenToClient(Self.Parent.ClientToScreen(Rect.TopLeft));
  Rect.BottomRight:=FSkinnedPanel.ScreenToClient(Self.Parent.ClientToScreen(Rect.BottomRight));
  Result:=FloatRect(Rect);
end; //GetLocationToPanel ******************************************************


(*******************************************************************************
  MakeBitmap32

  Make in Bitmap32 a TBitmap32 object with TBitmap Bmp and alpha channel AlphaBmp
*******************************************************************************)
procedure TSkinnedComponent.MakeBitmap32(var Bitmap32: TBitmap32; Bmp, AlphaBmp: TBitmap);
var alpha32: TBitmap32;
begin
  Bitmap32.Assign(Bmp);
  Bitmap32.ResetAlpha;

  if not AlphaBmp.Empty then
  begin
    alpha32:=TBitmap32.Create;
    alpha32.Assign(AlphaBmp);
    IntensityToAlpha(Bitmap32,alpha32);
    alpha32.Free;
  end;
end;

(*******************************************************************************
  MakeBitmap32

  Make in Bitmap32 a TBitmap32 object with TBitmap Bmp and alpha channel AlphaBmp (TBitmap32)
*******************************************************************************)
procedure TSkinnedComponent.MakeBitmap32(var Bitmap32: TBitmap32; Bmp:TBitmap; AlphaBmp: TBitmap32);
begin
  Bitmap32.Assign(Bmp);
  Bitmap32.ResetAlpha;

  if not AlphaBmp.Empty then
    IntensityToAlpha(Bitmap32,AlphaBmp);
end;

(*******************************************************************************
  InsertLayerToPanel(Bitmap: TBitmap)

  insert in associated SkinnedPanel a layer with Bitmap and alpha from FAlpha

  return: Layer (z-)index in SkinnedPanel
*******************************************************************************)
function TSkinnedComponent.InsertLayerToPanel(Bitmap: TBitmap; Alpha: TBitmap = nil):integer;
var bmp32: TBitmap32;
    AlphaBmp: TBitmap;
begin
  AlphaBmp:=TBitmap.Create;
  bmp32:=TBitmap32.Create;

  if Alpha<>nil then
    AlphaBmp.Assign(Alpha)  //if parameter alpha is set (=own alpha for this layer)
  else AlphaBmp.Assign(FAlpha);              //else take component alpha

  MakeBitmap32(bmp32, Bitmap, AlphaBmp);

  Result:=FSkinnedPanel.InsertBitmapLayer(bmp32, GetLocationToPanel);
  FSkinnedPanel.Image32.Layers[Result].OnHitTest:=PanelMouseHit;
  FSkinnedPanel.Image32.Layers[Result].OnMouseDown:=PanelMouseDown;
  FSkinnedPanel.Image32.Layers[Result].OnMouseUp:=PanelMouseUp;
  FSkinnedPanel.Image32.Layers[Result].OnMouseMove:=PanelMouseMove;

  AlphaBmp.Free;
end; //InsertLayerToPanel ******************************************************


(*******************************************************************************
  IsOverComponent(x,y)

  check if pixel (x/y) is part of Component or not (use for mouse-tests)
  return: true/false
*******************************************************************************)
function TSkinnedComponent.IsOverComponent(X,Y : integer):boolean;
var Col: TColor;
    Grey: integer;
    StretchFactorX: double;
    StretchFactorY: double;
begin
  if FAlpha.Empty then Result:=True
  else
  begin
    StretchFactorX:=FAlpha.Width/Self.Width;
    StretchFactorY:=FAlpha.Height/Self.Height;
    Col:= FAlpha.Canvas.Pixels[trunc(X*StretchFactorX),trunc(Y*StretchFactorY)]; //get color of pixel
    if Col=-1 then Col:=0;
    Grey:=(GetRValue(col)+GetGValue(col)+GetBValue(col)) div 3; //get intensity (greyscale)
    Result:=Grey>FOverComponentThreshold; //over Component if whiter than half grey
  end;
end; // IsOverComponent ********************************************************


(*******************************************************************************
  Loaded
*******************************************************************************)
procedure TSkinnedComponent.Loaded;
begin
  inherited Loaded;
end; //Loaded ******************************************************************


(*******************************************************************************
  Resize

  if a associated SkinnedPanel is assigned then change also the position of
  the four button bitmaps in layers of SkinnedPanel
*******************************************************************************)
procedure TSkinnedComponent.Resize;
begin
  inherited Resize;
end; //Resize ******************************************************************

//selfexplaining
procedure TSkinnedComponent.SetAlpha(bmp: TBitmap);
begin
  FAlpha.Assign(bmp);
end;

(*******************************************************************************
  SetEnabled

  enable/disable button and show appropriate bitmap
*******************************************************************************)
procedure TSkinnedComponent.SetEnabled(Value: boolean);
begin
  inherited SetEnabled(Value);
end; //SetEnabled **************************************************************


(*******************************************************************************
  RemoveFromPanel

  Remove component from SkinnedPanel and delete inserted layers
*******************************************************************************)
procedure TSkinnedComponent.RemoveFromPanel;
begin
  FSkinnedPanel.RemoveLayeredComponent(Self);
  MakePanelNotify;
end; //RemoveFromPanel *********************************************************

(*******************************************************************************
  InsertToPanel

  insert component to SkinnedPanel and insert layers
*******************************************************************************)
procedure TSkinnedComponent.InsertToPanel;
begin
  FSkinnedPanel.InsertLayeredComponent(Self);
end; //InsertToPanel ***********************************************************

(*******************************************************************************
  SetSkinnedPanel

  Set SkinnedPanel and insert component to layer and list in SkinnedPanel
*******************************************************************************)
procedure TSkinnedComponent.SetSkinnedPanel(NewPanel: TSkinnedPanel);
begin
  if FSkinnedPanel=NewPanel then Exit;

  if FSkinnedPanel<>nil then RemoveFromPanel;

  if NewPanel<>nil then
  begin
    FSkinnedPanel:=NewPanel;
    InsertToPanel;
  end;
end;


(*******************************************************************************
  PanelMouseMove

  This proc is assigend in InsertLayerToPanel to each layer of this button.
  A SkinnedPanel catches all mouse events and routes the "mouse move" event
  via PanelHitTest to this proc.
*******************************************************************************)
procedure TSkinnedComponent.PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var ComponentLoc: TFloatRect;
    MouseButton: TMouseButton;
begin
  if MouseEnter then
  begin
    Paint;
    if assigned(OnMouseEnter) then
    begin
      if ssMiddle in Shift then MouseButton:=mbMiddle;
      if ssRight in Shift then MouseButton:=mbRight;
      if ssLeft in Shift then MouseButton:=mbLeft;
      OnMouseEnter(Sender,  MouseButton, Shift, X-trunc(ComponentLoc.Left), Y-trunc(ComponentLoc.Top));  //call user on enter proc
    end;
    MouseEnter:=False;
  end;

  if MouseLeave then
  begin
    Paint;
    if assigned(OnMouseLeave) then
    begin
      if ssMiddle in Shift then MouseButton:=mbMiddle;
      if ssRight in Shift then MouseButton:=mbRight;
      if ssLeft in Shift then MouseButton:=mbLeft;
      OnMouseLeave(Sender,  MouseButton, Shift, X-trunc(ComponentLoc.Left), Y-trunc(ComponentLoc.Top));  //call user on Leave proc
    end;
    MouseLeave:=False;
  end;

  if MouseOver and Assigned(OnMouseMove) then
    OnMouseMove(Self, Shift, X-trunc(ComponentLoc.Left), Y-trunc(ComponentLoc.Top)) ;  //OnMouseMove only if mouse is over masked button with coords referring to button
end;


(*******************************************************************************
  MouseMove

  "normal" event proc in case button does not lie on a SkinnedPanel
*******************************************************************************)
procedure TSkinnedComponent.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  PanelMouseMove(Self, Shift, X, Y);
end;


(*******************************************************************************
  PanelMouseDown

  This proc is assigend in InsertLayerToPanel to each layer of this button.
  A SkinnedPanel catches all mouse events and routes the "mouse down" event
  via PanelHitTest to this proc.
*******************************************************************************)
procedure TSkinnedComponent.PanelMouseDown(Sender: TObject; Button: TMouseButton ; Shift: TShiftState; X, Y: Integer);
begin
  if assigned(OnMouseDown) then OnMouseDown(Self,Button,Shift,X,Y);
end;

(*******************************************************************************
  MouseDown

  "normal" event proc in case button does not lie on a SkinnedPanel
*******************************************************************************)
procedure TSkinnedComponent.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  PanelMouseDown(Self, Button, Shift, X, Y);
end;


(*******************************************************************************
  PanelMouseUp

  This proc is assigend in InsertLayerToPanel to each layer of this button.
  A SkinnedPanel catches all mouse events and routes the "mouse up" event
  via PanelHitTest to this proc.
*******************************************************************************)
procedure TSkinnedComponent.PanelMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if assigned(OnMouseUp) then OnMouseUp(Self,Button,Shift,X,Y);
end;


(*******************************************************************************
  MouseUp

  "normal" event proc in case button does not lie on a SkinnedPanel
*******************************************************************************)
procedure TSkinnedComponent.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  PanelMouseUp(Self, Button, Shift, X, Y);
end;

(*******************************************************************************
  Paint

  paint component
*******************************************************************************)
procedure TSkinnedComponent.Paint;
begin
  inherited Paint;
end;

(*******************************************************************************
  PanelMouseHit

  Check if mouse event on (x/y) over SkinnedPanel hits the button

  This event proc is assigned in InsertLayerToPanel to each button layer in SkinnedPanel
*******************************************************************************)
procedure TSkinnedComponent.PanelMouseHit(Sender: TObject; X, Y: Integer; var Passed: Boolean);
var ComponentLoc: TFloatRect;
    FNewMouseOver: Boolean;
begin
  Passed:=False;
  if not Enabled then Exit;  //don't react if not enabled!
  ComponentLoc:=GetLocationToPanel; //get location of button in panel coords

  if (X>=ComponentLoc.Left) and (X<=ComponentLoc.Right) and (Y>=ComponentLoc.Top) and (Y<=ComponentLoc.Bottom) then
  begin
    FNewMouseOver:=IsOverComponent(X-trunc(ComponentLoc.Left), Y-trunc(ComponentLoc.Top));
    if FNewMouseOver<>FMouseOver then //enter or leave component
    begin
      FMouseOver:=FNewMouseOver;
      if FMouseOver then FMouseEnter:=True
      else FMouseLeave:=True;
      Passed:=True; //pass if enter or leave component
    end;
    if FMouseOver then Passed:=True;  //pass if over component
  end
  else if FMouseOver then //if mouse is not over component but was last time -> mouse has left, set FMouseLeave and pass mouse event
  begin
    FMouseLeave:=True;
    FMouseOver:=False;
    Passed:=True;
  end;
end; //PanelMouseHit ***********************************************************


end.
