unit SkinnedImage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GR32_Image, GR32, GR32_Layers, GR32_Filters, SkinnedComponents;

type
  TSkinnedImage = class(TSkinnedComponent)
  private
    FFace: TBitmap;
    FFaceLayerIndex: integer;
    FDisabled: TBitmap;
    FDisabledLayerIndex: integer;
    FStretch: Boolean;
    procedure SetFace(bmp: TBitmap);                 //Set bitmaps
    procedure SetDisabled(bmp: TBitmap);
    procedure SetStretch(Value: Boolean);
  protected
    procedure SetEnabled(Value: boolean); override;
    procedure Resize; override;
    procedure Loaded; override;
    procedure PanelMouseDown(Sender: TObject; Button: TMouseButton ; Shift: TShiftState; X, Y: Integer); override;  //event procs for SkinnedPanel layer, assigned in PaintToPanel
    procedure PanelMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure CMMouseLeave(var Message: TMessage);
    procedure RemoveFromPanel; override;
    procedure InsertToPanel; override;
    procedure MakePanelNotify; override; //make panel notify its skinned component that layers has been removed
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure NotifyLayersRemoved(FromIndex, ToIndex: integer); override;
  published
    property Face: TBitmap read FFace write SetFace;
    property Disabled: TBitmap read FDisabled write SetDisabled;
    property Stretch: Boolean read FStretch write SetStretch;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('SkinnedComponents', [TSkinnedImage]);
end;

(*******************************************************************************
  Create

  constructor
  create other subcomponents:
  FFace, FDisabled
*******************************************************************************)
constructor TSkinnedImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FFace:=TBitmap.Create;
  FDisabled:=TBitmap.Create;
  FStretch:=True;
end; //Create ******************************************************************


(*******************************************************************************
  Destroy

  destructor
*******************************************************************************)
destructor TSkinnedImage.Destroy;
begin
  FFace.Free;
  FDisabled.Free;

  inherited Destroy;
end; //Destroy *****************************************************************


(*******************************************************************************
  Loaded

  if a associated SkinnedPanel is assigned then insert button in 4 layers
  (each one FFace, FHover, FDown and FDisabled in one)

  return: Layer (z-)index in SkinnedPanel
*******************************************************************************)
procedure TSkinnedImage.Loaded;
begin
  inherited;
end; //Loaded ******************************************************************


(*******************************************************************************
  Resize

  if a associated SkinnedPanel is assigned then change also the position of
  the four button bitmaps in layers of SkinnedPanel
*******************************************************************************)
procedure TSkinnedImage.Resize;
var newPos: TFloatRect;
begin
  inherited Resize;

  if not FStretch then
  begin
    Width:=FFace.Width;
    Height:=FFace.Height;
  end;

  if SkinnedPanel<>nil then
  begin
    newPos:=GetLocationToPanel;
    SkinnedPanel.SetLayerLocation(FFaceLayerIndex,newPos);
    SkinnedPanel.SetLayerLocation(FDisabledLayerIndex,newPos);
    SkinnedPanel.Invalidate;
  end;
end; //Resize ******************************************************************


(*******************************************************************************
  SetStretch

  set flag to stretch image
*******************************************************************************)
procedure TSkinnedImage.SetStretch(Value: Boolean);
begin
  if Value<>FStretch then
  begin
    FStretch:=Value;
    if not Value then
    begin
      Width:=FFace.Width;
      Height:=FFace.Height;
    end;
    Paint;
  end;
end;


//some selfexplaining setting functions

procedure TSkinnedImage.SetFace(bmp: TBitmap);
begin
  FFace.Assign(bmp);
end;

procedure TSkinnedImage.SetDisabled(bmp: TBitmap);
begin
  FDisabled.Assign(bmp);
end;

(*******************************************************************************
  SetEnabled

  enable/disable button and show appropriate bitmap
*******************************************************************************)
procedure TSkinnedImage.SetEnabled(Value: boolean);
begin
  if Value<>Enabled then
  begin
    if SkinnedPanel<>nil then
    begin
      SkinnedPanel.Image32.BeginUpdate;
      if Value then
      begin
        SkinnedPanel.Image32.Layers[FFaceLayerIndex].Visible:=True;
        SkinnedPanel.Image32.Layers[FDisabledLayerIndex].Visible:=False;
      end
      else
      begin
        SkinnedPanel.Image32.Layers[FFaceLayerIndex].Visible:=False;
        SkinnedPanel.Image32.Layers[FDisabledLayerIndex].Visible:=True;
      end;
      SkinnedPanel.Image32.EndUpdate;
      SkinnedPanel.Image32.Invalidate;
    end;
  end;

  inherited;
end; //SetEnabled **************************************************************


(*******************************************************************************
  RemoveFromPanel

  remove layers from associated SkinnedPanel
*******************************************************************************)
procedure TSkinnedImage.RemoveFromPanel;
begin
  with SkinnedPanel.Image32.Layers do
  begin
    if Count>=FDisabledLayerIndex then
    begin
      Delete(FDisabledLayerIndex);
      Delete(FFaceLayerIndex);
    end;
  end;

  inherited; //remove component from component list in SkinnedPanel (see TSkinnedComponent)
end;


(*******************************************************************************
  InsertToPanel

  insert layers to associated SkinnedPanel
*******************************************************************************)
procedure TSkinnedImage.InsertToPanel;
begin
  inherited; //insert component in component list in SkinnedPanel (see TSkinnedComponent)

  FFaceLayerIndex:=InsertLayerToPanel(FFace);
  FDisabledLayerIndex:=InsertLayerToPanel(FDisabled);
  //show face bitmap
  SkinnedPanel.Image32.Layers[FFaceLayerIndex].Visible:=True;
  SkinnedPanel.Image32.Layers[FDisabledLayerIndex].Visible:=False;
end;


(*******************************************************************************
  CMMouseLeave

  "normal" event proc in case button does not lie on a SkinnedPanel
*******************************************************************************)
procedure TSkinnedImage.CMMouseLeave(var Message: TMessage);
begin
  MouseOver:=False;
  Paint;
end;

(*******************************************************************************
  PanelMouseDown

  This proc is assigend in PaintToPanel to each layer of this button.
  A SkinnedPanel catches all mouse events and routes the "mouse down" event
  via PanelHitTest to this proc.
*******************************************************************************)
procedure TSkinnedImage.PanelMouseDown(Sender: TObject; Button: TMouseButton ; Shift: TShiftState; X, Y: Integer);
var ButtonLoc: TFloatRect;
begin
  ButtonLoc:=GetLocationToPanel;
  if MouseOver then inherited PanelMouseDown(Sender, Button, Shift, X-trunc(ButtonLoc.Left), Y-trunc(ButtonLoc.Top)) ;  //OnMouseDown only if mouse is over masked button with coords referring to button
end;


(*******************************************************************************
  PanelMouseUp

  This proc is assigend in PaintToPanel to each layer of this button.
  A SkinnedPanel catches all mouse events and routes the "mouse up" event
  via PanelHitTest to this proc.
*******************************************************************************)
procedure TSkinnedImage.PanelMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var ButtonLoc: TFloatRect;
begin
  ButtonLoc:=GetLocationToPanel;
  inherited PanelMouseUp(Sender, Button, Shift, X-trunc(ButtonLoc.Left), Y-trunc(ButtonLoc.Top)) ;  //OnMouseUp only if button has been pressed before; with coords referring to button
end;


(*******************************************************************************
  Paint

  paint button in correct state
*******************************************************************************)
procedure TSkinnedImage.Paint;
begin
  inherited;
end; //Paint *******************************************************************


(*******************************************************************************
  NotifyLayersRemoved

  is called from the associated SkinnedPanel in case another layered component
  has been removed and therefore layer indexes for this button has to be adjusted
*******************************************************************************)
procedure TSkinnedImage.NotifyLayersRemoved(FromIndex, ToIndex: integer);
var RemovedLayersCount: integer;
begin
  if ToIndex<FFaceLayerIndex then  //if major index of removed layers is less than minor layer index of this button then correct indexes
  begin
    RemovedLayersCount:=ToIndex-FromIndex+1;
    FFaceLayerIndex:=FFaceLayerIndex-RemovedLayersCount;
    FDisabledLayerIndex:=FDisabledLayerIndex-RemovedLayersCount;
  end;
end;


(*******************************************************************************
  MakePanelNotify

  make panel notify its skinned component that layers has been removed
*******************************************************************************)
procedure TSkinnedImage.MakePanelNotify;
begin
  SkinnedPanel.NotifyComponents(FFaceLayerIndex,FDisabledLayerIndex);
end;

end.
