unit SkinnedPBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GR32_Image, GR32, GR32_Layers, GR32_Filters, SkinnedComponents;

type
  TSkinnedPBar = class(TSkinnedComponent)
  private
    FBackground: TBitmap;
    FBackgroundLayerIndex: integer;
    FBar: TBitmap;
    FBarLayerIndex: integer;
    FBarAlpha: TBitmap;
    FDisabled: TBitmap;
    FDisabledLayerIndex: integer;
    FPercent: Double;
    FStretchBar: Boolean;
    procedure SetBackground(bmp: TBitmap);                 //Set bitmaps
    procedure SetBar(bmp: TBitmap);
    procedure SetBarAlpha(bmp: TBitmap);
    procedure SetDisabled(bmp: TBitmap);
    procedure SetPercent(Value: double);
    procedure SetStretchBar(Value: Boolean);
    procedure SetBarBitmap(bmp, alpha: TBitmap); overload;
    procedure SetBarBitmap(bmp:TBitmap; alpha: TBitmap32); overload;
  protected
    procedure SetEnabled(Value: boolean); override;
    procedure Resize; override;
    procedure Loaded; override;
    procedure PanelMouseDown(Sender: TObject; Button: TMouseButton ; Shift: TShiftState; X, Y: Integer); override;  //event procs for SkinnedPanel layer, assigned in PaintToPanel
    procedure PanelMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure CMMouseLeave(var Message: TMessage);
    procedure RemoveFromPanel; override;
    procedure InsertToPanel; override;
    procedure MakePanelNotify; override; //make panel notify its skinned component that layers has been removed
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure NotifyLayersRemoved(FromIndex, ToIndex: integer); override;
  published
    property Background: TBitmap read FBackground write SetBackground;
    property Bar: TBitmap read FBar write SetBar;
    property BarAlpha: TBitmap read FBarAlpha write SetBarAlpha;
    property Disabled: TBitmap read FDisabled write SetDisabled;
    property Position: double read FPercent write SetPercent;
    property StretchBar: boolean read FStretchBar write SetStretchBar default false;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('SkinnedComponents', [TSkinnedPBar]);
end;

(*******************************************************************************
  Create

  constructor
  create other subcomponents:
  FBackground, FBar, FDisabled
*******************************************************************************)
constructor TSkinnedPBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FBackground:=TBitmap.Create;
  FBar:=TBitmap.Create;
  FBarAlpha:=TBitmap.Create;
  FDisabled:=TBitmap.Create;
end; //Create ******************************************************************


(*******************************************************************************
  Destroy

  destructor
*******************************************************************************)
destructor TSkinnedPBar.Destroy;
begin
  FBackground.Free;
  FBar.Free;
  FBarAlpha.Free;
  FDisabled.Free;

  inherited Destroy;
end; //Destroy *****************************************************************


(*******************************************************************************
  Loaded
*******************************************************************************)
procedure TSkinnedPBar.Loaded;
begin
  inherited;

end; //Loaded ******************************************************************


(*******************************************************************************
  Resize

  if a associated SkinnedPanel is assigned then change also the position of
  the four button bitmaps in layers of SkinnedPanel
*******************************************************************************)
procedure TSkinnedPBar.Resize;
var newPos: TFloatRect;
begin
  inherited Resize;

  if SkinnedPanel<>nil then
  begin
    newPos:=GetLocationToPanel;
    SkinnedPanel.SetLayerLocation(FBackgroundLayerIndex,newPos);
    SkinnedPanel.SetLayerLocation(FBarLayerIndex,newPos);
    SkinnedPanel.SetLayerLocation(FDisabledLayerIndex,newPos);
    SkinnedPanel.Invalidate;
  end;
end; //Resize ******************************************************************

//some selfexplaining setting functions

procedure TSkinnedPBar.SetBackground(bmp: TBitmap);
begin
  FBackground.Assign(bmp);
end;

procedure TSkinnedPBar.SetBar(bmp: TBitmap);
begin
  FBar.Assign(bmp);
end;

procedure TSkinnedPBar.SetBarAlpha(bmp: TBitmap);
begin
  FBarAlpha.Assign(bmp);
end;


procedure TSkinnedPBar.SetDisabled(bmp: TBitmap);
begin
  FDisabled.Assign(bmp);
end;

(*******************************************************************************
  SetPercent

  set percentage of progressbar
*******************************************************************************)
procedure TSkinnedPBar.SetPercent(Value: double);
begin
  if Value<>FPercent then
  begin
    FPercent:=Value;
    if FPercent>1 then FPercent:=1;
    if FPercent<0 then FPercent:=0;
    Paint;
  end;
end;

(*******************************************************************************
  SetBarBitmap (with two TBitmap objects)

  set bar bitmap in SkinnedPanel layer that is already initialised
*******************************************************************************)
procedure TSkinnedPBar.SetBarBitmap(bmp, alpha: TBitmap);
var bmp32: TBitmap32;
begin
  bmp32:=TBitmap32.Create;
  bmp32.DrawMode:=dmBlend;
  bmp32.MasterAlpha:=255;
  bmp32.StretchFilter:=sfMitchell;

  with SkinnedPanel.Image32 do
  begin
    MakeBitmap32(bmp32, bmp, alpha);
    BeginUpdate;
    TBitmapLayer(Layers[FBarLayerIndex]).Bitmap.Assign(bmp32);
    EndUpdate;
    bmp32.Free;
  end;
end;


(*******************************************************************************
  SetBarBitmap (with one TBitmap and alpha as TBitmap32)

  set bar bitmap in SkinnedPanel layer that is already initialised
*******************************************************************************)
procedure TSkinnedPBar.SetBarBitmap(bmp:TBitmap; alpha: TBitmap32);
var bmp32: TBitmap32;
begin
  bmp32:=TBitmap32.Create;
  bmp32.DrawMode:=dmBlend;
  bmp32.MasterAlpha:=255;
  bmp32.StretchFilter:=sfMitchell;

  with SkinnedPanel.Image32 do
  begin
    MakeBitmap32(bmp32, bmp, alpha);
    BeginUpdate;
    TBitmapLayer(Layers[FBarLayerIndex]).Bitmap.Assign(bmp32);
    EndUpdate;
    SkinnedPanel.Image32.Repaint;
    bmp32.Free;
  end;
end;


(*******************************************************************************
  SetStretchBar

  set flag to stretch or clip bar
*******************************************************************************)
procedure TSkinnedPBar.SetStretchBar(Value: Boolean);
begin
  if Value<>FStretchBar then
  begin
    FStretchBar:=Value;
    if Value and assigned(SkinnedPanel) then SetBarBitmap(FBar, FBarAlpha); //stretch bar -> recreate bar bitmap
    Paint;
  end;
end;


(*******************************************************************************
  SetEnabled

  enable/disable pbar and show appropriate bitmap
*******************************************************************************)
procedure TSkinnedPBar.SetEnabled(Value: boolean);
begin
  if Value<>Enabled then
  begin
    if SkinnedPanel<>nil then
    begin
      SkinnedPanel.Image32.BeginUpdate;
      if Value then
      begin
        SkinnedPanel.Image32.Layers[FBackgroundLayerIndex].Visible:=True;
        SkinnedPanel.Image32.Layers[FBarLayerIndex].Visible:=True;
        SkinnedPanel.Image32.Layers[FDisabledLayerIndex].Visible:=False;
      end
      else
      begin
        SkinnedPanel.Image32.Layers[FBackgroundLayerIndex].Visible:=False;
        SkinnedPanel.Image32.Layers[FBarLayerIndex].Visible:=False;
        SkinnedPanel.Image32.Layers[FDisabledLayerIndex].Visible:=True;
      end;
      SkinnedPanel.Image32.EndUpdate;
      SkinnedPanel.Image32.Invalidate;
    end;
  end;

  inherited;
end; //SetEnabled **************************************************************


(*******************************************************************************
  RemoveFromPanel

  remove layers from associated SkinnedPanel
*******************************************************************************)
procedure TSkinnedPBar.RemoveFromPanel;
begin
  with SkinnedPanel.Image32.Layers do
  begin
    if Count>=FDisabledLayerIndex then
    begin
      Delete(FDisabledLayerIndex);
      Delete(FBarLayerIndex);
      Delete(FBackgroundLayerIndex);
    end;
  end;

  inherited; //remove component from component list in SkinnedPanel (see TSkinnedComponent)
end;


(*******************************************************************************
  InsertToPanel

  insert layers to associated SkinnedPanel
*******************************************************************************)
procedure TSkinnedPBar.InsertToPanel;
begin
  inherited; //insert component in component list in SkinnedPanel (see TSkinnedComponent)

  FBackgroundLayerIndex:=InsertLayerToPanel(FBackground);
  FBarLayerIndex:=InsertLayerToPanel(FBar, FBarAlpha);     //insert bar bitmap with own alpha channel
  FDisabledLayerIndex:=InsertLayerToPanel(FDisabled);
  //show background and bar bitmap
  SkinnedPanel.Image32.Layers[FBackgroundLayerIndex].Visible:=True;
  SkinnedPanel.Image32.Layers[FBarLayerIndex].Visible:=True;
  SkinnedPanel.Image32.Layers[FDisabledLayerIndex].Visible:=False;
end;


(*******************************************************************************
  CMMouseLeave

  "normal" event proc in case button does not lie on a SkinnedPanel
*******************************************************************************)
procedure TSkinnedPBar.CMMouseLeave(var Message: TMessage);
begin
  MouseOver:=False;
  Paint;
end;


(*******************************************************************************
  PanelMouseDown

  This proc is assigend in PaintToPanel to each layer of this button.
  A SkinnedPanel catches all mouse events and routes the "mouse down" event
  via PanelHitTest to this proc.
*******************************************************************************)
procedure TSkinnedPBar.PanelMouseDown(Sender: TObject; Button: TMouseButton ; Shift: TShiftState; X, Y: Integer);
var ComponentLoc: TFloatRect;
begin
  ComponentLoc:=GetLocationToPanel;
  if MouseOver then inherited PanelMouseDown(Sender, Button, Shift, X-trunc(ComponentLoc.Left), Y-trunc(ComponentLoc.Top)) ;  //OnMouseDown only if mouse is over masked button with coords referring to button
end;


(*******************************************************************************
  PanelMouseUp

  This proc is assigend in PaintToPanel to each layer of this button.
  A SkinnedPanel catches all mouse events and routes the "mouse up" event
  via PanelHitTest to this proc.
*******************************************************************************)
procedure TSkinnedPBar.PanelMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var ComponentLoc: TFloatRect;
begin
  ComponentLoc:=GetLocationToPanel;
  inherited PanelMouseUp(Sender, Button, Shift, X-trunc(ComponentLoc.Left), Y-trunc(ComponentLoc.Top)) ;  //OnMouseUp only if button has been pressed before; with coords referring to button
end;

(*******************************************************************************
  Paint

  paint pbar in correct state
*******************************************************************************)
procedure TSkinnedPBar.Paint;
var BarLoc: TFloatRect;
    ClipWidth: integer;
    ClippedAlpha: TBitmap32;
begin
  inherited;

  if not FBar.Empty and assigned(SkinnedPanel) and Enabled then
  begin
    //percentage als text schreiben!
    BarLoc:=GetLocationToPanel;
    if StretchBar then BarLoc.Right:=BarLoc.Left+(BarLoc.Right-BarLoc.Left)*FPercent //stretch entire bar bitmap to percentage range
    else  //clip bar bitmap to percentage range
    begin
      ClipWidth:=trunc(FBarAlpha.Width*FPercent);  //new width of bar
      ClippedAlpha:=TBitmap32.Create;
      ClippedAlpha.Assign(FBarAlpha);
      //clip region of bar not to be shown
      ClippedAlpha.FillRect(ClipWidth,0,FBarAlpha.Width,FBarAlpha.Height,RGB(0,0,0));
      SetBarBitmap(FBar, ClippedAlpha); //set bar but clipped (via alpha channel)
    end;
    SkinnedPanel.SetLayerLocation(FBarLayerIndex,BarLoc);           //resize bar layer
    SkinnedPanel.Image32.Invalidate;
  end;
end; //Paint *******************************************************************


(*******************************************************************************
  NotifyLayersRemoved

  is called from the associated SkinnedPanel in case another layered component
  has been removed and therefore layer indexes for this button has to be adjusted
*******************************************************************************)
procedure TSkinnedPBar.NotifyLayersRemoved(FromIndex, ToIndex: integer);
var RemovedLayersCount: integer;
begin
  if ToIndex<FBackgroundLayerIndex then  //if major index of removed layers is less than minor layer index of this button then correct indexes
  begin
    RemovedLayersCount:=ToIndex-FromIndex+1;
    FBackgroundLayerIndex:=FBackgroundLayerIndex-RemovedLayersCount;
    FBarLayerIndex:=FBarLayerIndex-RemovedLayersCount;
    FDisabledLayerIndex:=FDisabledLayerIndex-RemovedLayersCount;
  end;
end;


(*******************************************************************************
  MakePanelNotify

  make panel notify its skinned component that layers has been removed
*******************************************************************************)
procedure TSkinnedPBar.MakePanelNotify;
begin
  SkinnedPanel.NotifyComponents(FBackgroundLayerIndex,FDisabledLayerIndex);
end;

end.
